""" aws_with module definition """

from . import utils
from . import regions
from . import organizations
from . import commands
from . import workplan
from . import output
from . import cli
from . import main

VERSION = "0.9.1"
BUILD_TIME = "2017-08-27 11:14:00"
BUILD_COMMIT_HASH = "0012300400g0hf020304f34f"
