from setuptools import setup
import aws_with

setup(
   name='aws_with',
   version=aws_with.VERSION,
   description='Command line utility to help manage AWS accounts, regions and organizations',
   license="APACHE2",
   long_description=open("README.rst").read(),
   author='Mike Evans',
   author_email='miev@amazon.com',
   packages=['aws_with'],
   install_requires=["boto3", "PyYAML"],
   platforms='any',
   data_files = [("", ["LICENSE.txt"])],
   entry_points={'console_scripts': ['aws_with = aws_with.main:main']},
   classifiers=['Development Status :: 4 - Beta',
                'Environment :: Console',
                'Intended Audience :: Developers',
                'Intended Audience :: End Users/Desktop',
                'Intended Audience :: Information Technology',
                'Intended Audience :: System Administrators',
                'License :: OSI Approved :: Apache Software License',
                'Operating System :: POSIX',
                'Operating System :: Microsoft :: Windows',
                'Operating System :: MacOS :: MacOS X',
                'Topic :: System :: Systems Administration',
                'Topic :: Utilities',
                'Programming Language :: Python',
                'Programming Language :: Python :: 2.6',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3.4',
                'Programming Language :: Python :: 3.5',
                'Programming Language :: Python :: 3.6',
                'Programming Language :: Python :: 3.7'],
)

